#ifndef __ASYNCTASKS_H__
#define __ASYNCTASKS_H__

#include "driver/spi_master.h"
#include "driver/gpio.h"
#include "debounce.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"


typedef struct {
    QueueHandle_t queue;
    EventGroupHandle_t eventgroup;
    uint8_t bitmap[TOT_OUTPUTS];
    uint8_t inputbitmap[TOT_INPUTS];
    uint16_t timer;
    uint16_t timehigh, timelow;
    int impulsi;
    int starter;

    int8_t dac;
    uint16_t time1, time2;
    uint8_t scaletime1, scaletime2;
    uint16_t vdi1, vdi2, vdf1, vdf2;

    int num420ma;
    int limit420ma;
    int direction;

    char    f_dac;
    uint8_t dacstart, dacfinish;
    uint16_t first420limit;
} timer_args_t;

void launch_simple_timer_task(uint16_t timer, int alarm, uint8_t limit420ma, int8_t num420ma, int direction, uint8_t *bitmap, QueueHandle_t queue, EventGroupHandle_t group);
void launch_simple_simple_timer_task(uint16_t timer, int alarm, uint8_t *bitmap, QueueHandle_t queue, EventGroupHandle_t group);
void launch_timer_task(uint16_t timer, int alarm, uint8_t limit420ma, int8_t num420ma, int direction, uint8_t numdac, 
    uint8_t dacstart, uint8_t dacfinish, uint8_t first420limit, uint8_t *bitmap, QueueHandle_t queue, EventGroupHandle_t group);
void launch_pwm_timer_task(uint16_t high, uint16_t low, int impulsi, uint8_t *bitmap, QueueHandle_t queue);
void init_async_process_management();
void get_output_timers(int num, uint16_t *timers);
void delete_all_tasks();
void delete_tasks_by_alarm(uint8_t *bitmap, uint16_t *ma420);
void delete_tasks_by_output(uint8_t *bitmap);
void launch_scaled_dac_timer_task(uint16_t timer1, uint16_t timer2, int alarm, uint8_t dac, 
        uint8_t vdi1, uint8_t vdf1, uint8_t vdi2, uint8_t vdf2, uint8_t scaletime1, uint8_t scaletime2,
        uint8_t limit420ma, int8_t num420ma, int direction, uint8_t *bitmap, QueueHandle_t queue);
void launch_timer_after_ma420_task(uint16_t timer, uint8_t limit420ma, int8_t num420ma, int direction,
    uint8_t first420limit, uint8_t *inputbitmap, uint8_t *outputbitmap, QueueHandle_t queue, EventGroupHandle_t group);
void launch_timer_after_input_task(uint16_t timer, int starter, uint8_t limit420ma, int8_t num420ma, int direction,
        uint8_t *inputbitmap, uint8_t *outputbitmap, QueueHandle_t queue, EventGroupHandle_t group);

#endif
