#ifndef __CONFIGURATION_H__
#define __CONFIGURATION_H__
#include "hardwareprofile.h"

#define NUM_SLAVES 2
#define NUM_PRESS 2
#define NUM_TEMP 2
#define NUM_420 2
#define NUM_ADC 2
#define NUM_DAC 2
#define NUM_EXT_PRESS 2
#define NUM_EXT_TEMP 2
#define NUM_EXT_420 4
#define NUM_EXT_ADC 2
#define NUM_EXT_DAC 2
#define NUM_OUTPUTS 1
#define NUM_INPUTS 1
#define NUM_EXT_OUTPUTS 2
#define NUM_EXT_INPUTS 1
#define TOT_OUTPUTS (NUM_OUTPUTS + NUM_EXT_OUTPUTS * NUM_SLAVES)
#define TOT_INPUTS (NUM_INPUTS + NUM_EXT_INPUTS * NUM_SLAVES)
#define TOT_PRESS (NUM_PRESS + NUM_EXT_PRESS * NUM_SLAVES)
#define TOT_TEMP (NUM_TEMP + NUM_EXT_TEMP * NUM_SLAVES)
#define TOT_420 (NUM_420 + NUM_EXT_420 * NUM_SLAVES)
#define TOT_ADC (NUM_ADC + NUM_EXT_ADC * NUM_SLAVES)
#define TOT_DAC (NUM_DAC + NUM_EXT_DAC * NUM_SLAVES)
#define TOT_EXT_OUTPUTS (NUM_EXT_OUTPUTS * NUM_SLAVES)
#define TOT_EXT_INPUTS (NUM_EXT_INPUTS * NUM_SLAVES)
#define TOT_EXT_PRESS (NUM_EXT_PRESS * NUM_SLAVES)
#define TOT_EXT_TEMP (NUM_EXT_TEMP * NUM_SLAVES)
#define TOT_EXT_420 (NUM_EXT_420 * NUM_SLAVES)
#define TOT_EXT_ADC (NUM_EXT_ADC * NUM_SLAVES)
#define TOT_EXT_DAC (NUM_EXT_DAC * NUM_SLAVES)

typedef enum {
    NERO = 0,
    VERDE,
    ROSSO,
    GIALLO,
    BLU,
    AZZURRO,
    MAGENTA,
    BIANCO,
} colori_led_t;

typedef struct {
    uint8_t      outputs[TOT_OUTPUTS];
    uint8_t      inputs[TOT_INPUTS];
    uint8_t      specialinputs[TOT_INPUTS];
    uint8_t      invertedinputs[TOT_INPUTS];
    uint8_t      pressostati;
    uint8_t      temperature;
    uint8_t      ad420[2];
    uint8_t      adc;
    uint8_t      dac;
    int          alarm;
    colori_led_t colore[NUM_SLAVES];
    uint16_t     offsetpress[TOT_PRESS];
    uint8_t      reset_outputs;
    uint8_t      pulsing_led;
    uint8_t      f_pulsing_led;
} machine_configuration_t;

void default_configuration(machine_configuration_t *conf);
void unpack_configuration(machine_configuration_t *conf, uint8_t *buffer);
int  pack_configuration(machine_configuration_t conf, uint8_t *buffer);
int is_slave_configured(machine_configuration_t *conf, int slave);
void turn_on_led(machine_configuration_t conf, uint8_t *bitmap);
#endif