#ifndef __PACKET_MANAGER_H__
#define __PACKET_MANAGER_H__
#include <stdint.h>

#define PREAMBLE_LEN 2
#define HEADER_LEN 8
#define PACKET_BUFFER_SIZE 512
#define TIMEOUT_US 1000UL

#define     SLAVE_STATE_LEN 24
#define     SLAVE_HEADER    2

#define     GET_CONFIG      0x0A00
#define     SET_CONFIG      0x1A00

#define     GET_REV_SCHEDA  0x0B00
#define     SET_REV_SCHEDA  0x1B00

#define     RESET_0         0x1F00
#define     LCD_ONOFF       0x1C00
#define     GET_RESET_0     0x0F01
#define     SET_RESET_0     0x1F01

#define     GET_STATE       0x1000
#define     GET_INPUTS      0x1001
#define     GET_OUTPUTS     0x1002
#define     GET_TEMPS       0x1003
#define     GET_PRESS       0x1004

#define     GET_OUTPUTS1_TIME    0x1020
#define     GET_OUTPUTS2_TIME    0x1021
#define     GET_OUTPUTS3_TIME    0x1022
#define     GET_OUTPUTS4_TIME    0x1023
#define     GET_OUTPUTS5_TIME    0x1024

#define     GET_ADC         0x1050
#define     GET_420MA         0x1051

#define     SET_OUTPUTS     0x2000
#define     SET_OUTPUTn_TIME     0x2001
#define     SET_OUTPUTn_PWM     0x2002
#define     SET_OUTPUTS_TIME     0x2003
#define     SET_OUTPUTS_PWM     0x2004

#define     SET_OUTPUTn_TIME_420MA     0x2010
#define     SET_OUTPUTS_TIME_420MA     0x2011
#define     SET_OUTPUTn_SCALED_DAC  0x2012
#define     SET_OUTPUTS_SCALED_DAC  0x2013
#define     SET_OUTPUTn_SCALED_DAC_ALARM  0x2014
#define     SET_OUTPUTn_SCALED_DAC_420MA  0x2015
#define     SET_OUTPUTn_SCALED_DAC_420MA_ALARM  0x2016
#define     SET_OUTPUTn_TIME_420MA_DAC_REACT  0x2017
#define     SET_OUTPUTS_TIME_420MA_DAC_REACT  0x2018
#define     SET_OUTPUTn_TIME_AFTER_420MA  0x2019
#define     SET_OUTPUTS_TIME_AFTER_420MA  0x201D
#define     SET_OUTPUTn_TIME_AFTER_INPUT  0x201E

#define     SET_SINGLE_DA   0x2020
#define     SET_MULTI_DAC   0x2021

#define     SET_OUTPUTn_ALARM   0x201A
#define     SET_OUTPUTS_ALARM   0x201F
#define     RESET_PORT          0x201B
#define     RESET_PORT_BITMAP   0x201C

#define     SET_COLORE_LED_DEFAULT      0x2060
#define     SET_COLORE_LED      0x2061
#define     GET_COLORE_LED      0x1006
#define     SET_PULSING_LED      0x2062

#define     TAKE_OFFSET_PRESS   0x2008

#define     RESTART_BOARD       0x8000


//ERRORI

#define     ACK             0x00
#define     WRONG_CRC       0x01
#define     WRONG_FORMAT    0x02
#define     UNKNOWN_CMD     0x03
#define     IMPOSSIBLE      0x04
#define     QUEUE_FULL      0x05

#define     COMMAND_NACK(x)     (x+0x8000)

typedef struct {
    int      lenght;
    uint16_t receiver;
    uint16_t sender;
    union {
        struct _cmd {
            uint8_t command;
            uint8_t subcommand;
        } parts;
        uint16_t cmd;
    } command;

    uint8_t crc;
    uint8_t data[255];
} packet_t;

int new_packet_data(unsigned long lasttimestamp, unsigned long timestamp, uint8_t *buffer, int *bufindex,
                    uint8_t *newdata, int len, int max);
int elaborate_data(uint8_t *data, int *start, int len, packet_t *packet);
int build_packet(uint16_t receiver, uint16_t sender, uint16_t command, uint8_t *data, int len, uint8_t *buffer);
int validate_slave_state(uint8_t *buffer, int len, uint8_t add);


#endif
