#include "hardwareprofile.h"
#include "adc_spi.h"
#include "driver/spi_master.h"
#include "system.h"
#include <string.h>

void init_adc(spi_device_handle_t *spi) {
    esp_err_t ret;
    uint8_t nullify[64] = {0};

    spi_device_interface_config_t devcfg = {
        .clock_speed_hz = SPI_FREQ_SLOW,     // Clock out at 1 MHz
        .mode           = 0,                 // SPI mode 0
        .spics_io_num   = ADC_CS,            // CS pin
        .queue_size     = 8,                 // We want to be able to queue 7 transactions at a time
        //.pre_cb=lcd_spi_pre_transfer_callback,  //Specify pre-transfer callback to handle D/C line
    };
    // Attach the LCD to the SPI bus
    ret = spi_bus_add_device(HSPI_HOST, &devcfg, spi);
    ESP_ERROR_CHECK(ret);

    spi_data(*spi, nullify, 64, -1);
}

uint16_t get_adc_channel(spi_device_handle_t spi, adc_channel_t channel) {
    esp_err_t         ret;
    spi_transaction_t t;
    uint16_t          res;
    memset(&t, 0, sizeof(t));      // Zero out the transaction
    t.length     = 8*3;// 4 command bits + 1 null bit + 12 ADC
    t.tx_data[0] = 0xC0 | (channel << 3);
    t.flags      = SPI_TRANS_USE_RXDATA | SPI_TRANS_USE_TXDATA;
    spi_device_acquire_bus(spi, portMAX_DELAY);
    ret          = spi_device_polling_transmit(spi, &t);     // Transmit!
    spi_device_release_bus(spi);
    ESP_ERROR_CHECK(ret);

    res = t.rx_data[0] << 11 | t.rx_data[1] << 3 | t.rx_data[2] >> 5;
    return res;
}

uint8_t adc_to_temp(uint16_t adc) {
    return (adc*172)/4095;
}

uint16_t ma420_to_adc(uint8_t ma) {
    uint32_t tmp;
    if (ma < 40)
        return 0;
    else if (ma > 200)
        return 200;
    
    tmp = (uint32_t)ma*1023;
    return (uint16_t)(tmp/200);
}

uint8_t adc_to_420ma(uint16_t adc) {
    uint32_t tmp;
    uint8_t result;
    tmp = (uint32_t)adc*200;
    result = (uint8_t)(tmp/1023);
    if (result >= 18 && result <= 22)
        result = 200;
    return result;
}

uint8_t adc_to_10v(uint16_t adc) {
    uint32_t tmp;
    tmp = (uint32_t)adc*100;
    return (uint8_t)(tmp/4095);
}

uint8_t adc_to_bar(uint16_t adc, uint16_t offset) {
    uint32_t tmp;
    adc = adc < offset ? 0 : adc - offset;
    adc = adc > 3722 ? 3722 : adc;
    // Proporzione perche' il valore e' tra 0 e 3V, non 3v3
    tmp = (uint32_t)adc*100;
    return (uint8_t)(tmp/(3722-offset));
}