#include <string.h>
#include "hardwareprofile.h"
#include "configuration.h"
#include "utility.h"

int is_slave_configured(machine_configuration_t *conf, int slave) {
    int start = NUM_OUTPUTS + NUM_EXT_OUTPUTS*slave, i;

    for (i = start; i < TOT_OUTPUTS; i++)
        if (conf->outputs[i] > 0)
            return 1;

    start = NUM_INPUTS + NUM_EXT_INPUTS*slave;
    for (i = start; i < TOT_INPUTS; i++)
        if (conf->inputs[i] > 0)
            return 1;

    if (((conf->pressostati >> (NUM_PRESS+slave*NUM_EXT_PRESS)) & 0x3) > 0)
        return 1;
    if (((conf->temperature >> (NUM_TEMP+slave*NUM_EXT_TEMP)) & 0x3) > 0)
        return 1;
    if (((conf->adc >> (NUM_ADC+slave*NUM_EXT_ADC)) & 0x3) > 0)
        return 1;
            //TODO: aggiungi il dac nel comando set config
    //if (((conf->dac >> (NUM_DAC+slave*NUM_EXT_DAC)) & 0x3) > 0)
        //return 1;

    if ((slave == 0 && (conf->ad420[1] >> NUM_420) & 0xF) > 0)
        return 1;
    else if (slave == 1 && (((conf->ad420[1] >> (NUM_420+NUM_EXT_420)) & 0xF) > 0 
                || (conf->ad420[0] & 0x3) > 0))
        return 1;

    return 0;
}

void default_configuration(machine_configuration_t *conf) {
    int i;

    conf->inputs[0] = 0xFF;
    conf->outputs[0] = 0xFF;

    conf->inputs[1] = 0xFF;
    conf->inputs[2] = 0x00;

    conf->outputs[1] = 0xFF;
    conf->outputs[2] = 0xFF;
    conf->outputs[3] = 0x00;
    conf->outputs[4] = 0x00;

    for (i = 0; i < TOT_INPUTS; i++)
        conf->specialinputs[i] = 0x00;
    for (i = 0; i < TOT_INPUTS; i++)
        conf->invertedinputs[i] = 0x00;

    conf->pressostati = 0x0f;
    conf->temperature = 0x0f;
    conf->ad420[0] = 0x00;
    conf->ad420[1] = 0x3f;
    conf->adc = 0x0f;
    conf->dac = 0x0f;
    conf->alarm = 0;
    conf->pulsing_led = 9;
    conf->f_pulsing_led = 0;
    memset(conf->offsetpress, 0, sizeof(uint16_t)*TOT_PRESS);
}

int pack_configuration(machine_configuration_t conf, uint8_t *buffer) {
    int i, j;
    j = 0;
    for (i = 0; i < TOT_INPUTS; i++)
        buffer[j++] = conf.inputs[i];
    for (i = 0; i < TOT_INPUTS; i++)
        buffer[j++] = conf.specialinputs[i];
    for (i = 0; i < TOT_INPUTS; i++)
        buffer[j++] = conf.invertedinputs[i];
    for (i = 0; i < TOT_OUTPUTS; i++)
        buffer[j++] = conf.outputs[i];

    buffer[j++] = conf.temperature;
    buffer[j++] = conf.pressostati;
    buffer[j++] = conf.adc;
    buffer[j++] = conf.ad420[0];
    buffer[j++] = conf.ad420[1];
    return j;
}

void unpack_configuration(machine_configuration_t *conf, uint8_t *buffer) {
    int i, j;
    j = 0;
    for (i = 0; i < TOT_INPUTS; i++)
        conf->inputs[i] = buffer[j++];
    for (i = 0; i < TOT_INPUTS; i++)
        conf->specialinputs[i] = buffer[j++];
    for (i = 0; i < TOT_INPUTS; i++)
        conf->invertedinputs[i] = buffer[j++];
    for (i = 0; i < TOT_OUTPUTS; i++)
        conf->outputs[i] = buffer[j++];

    conf->temperature = buffer[j++];
    conf->pressostati = buffer[j++];
    conf->adc = buffer[j++];
    conf->ad420[0] = buffer[j++];
    conf->ad420[1] = buffer[j++];
}

void turn_on_led(machine_configuration_t conf, uint8_t *bitmap) {
    if (conf.f_pulsing_led || conf.pulsing_led >= TOT_OUTPUTS*8)
        return;

    set_single_bitmap(conf.pulsing_led, bitmap);
}