#include <string.h>
#include "hardwareprofile.h"
#include "driver/spi_master.h"
#include "system.h"
#include "dac_spi.h"


void init_dac(spi_device_handle_t *spi) {
    esp_err_t ret;
    uint8_t   nullify[64];

    spi_device_interface_config_t devcfg = {
        .clock_speed_hz = SPI_FREQ_FAST,     // Clock out at 1 MHz
        .mode           = 0,                 // SPI mode 0
        .spics_io_num   = DAC_CS,            // CS pin
        .queue_size     = 8,                 // We want to be able to queue 7 transactions at a time
        //.pre_cb=lcd_spi_pre_transfer_callback,  //Specify pre-transfer callback to handle D/C line
    };
    // Attach the LCD to the SPI bus
    ret = spi_bus_add_device(HSPI_HOST, &devcfg, spi);
    ESP_ERROR_CHECK(ret);

    spi_data(*spi, nullify, 64, -1);
}

void set_dac_channel(spi_device_handle_t spi, dac_channel_t channel, uint16_t value) {
    esp_err_t         ret;
    spi_transaction_t t;
    uint16_t buffer;

    buffer = channel << 2 | WRITE_UPDATE; // CH e mode
    buffer = buffer << 10;
    buffer = buffer + value;
    buffer = buffer << 2;
    // Secondo il datasheet il pacchetto dovrebbe essere composto cosi', ma
    // il dispositivo sembra aver bisogno di un bit in piu' all'inizio per
    // cominciare a leggere il dato.
    buffer = buffer >> 1;

    memset(&t, 0, sizeof(t));     // Zero out the transaction
    t.length     = 8 * 2;         // 2 for channel select + 2 for command + 10 for value
    t.tx_data[0] = buffer >> 8;
    t.tx_data[1] = buffer & 0xFF;
    t.flags      = SPI_TRANS_USE_TXDATA;
    spi_device_acquire_bus(spi, portMAX_DELAY);
    ret          = spi_device_polling_transmit(spi, &t);     // Transmit!
    spi_device_release_bus(spi);
    ESP_ERROR_CHECK(ret);
}

uint16_t volts_to_dac(int volts) {
    return (volts*1023)/100;
}

uint8_t dac_to_volts(uint16_t dac) {
    return (dac*100)/1023;
}
