#include <string.h>
#include "hardwareprofile.h"
#include "system.h"
#include "input_expander.h"


void init_input_expander(spi_device_handle_t *spi, uint8_t add_spi_device) {
    esp_err_t ret;
    uint8_t   inputdir[3]  = {IN_EXPANDER_ADD, IN_EXPANDER_DIR_ADD, 0xFF};
    uint8_t   invertpol[3] = {IN_EXPANDER_ADD, IN_EXPANDER_POL_ADD, 0xFF};
    uint8_t   nullify[64]  = {0};

    if (add_spi_device) {
        spi_device_interface_config_t devcfg = {
            .clock_speed_hz = SPI_FREQ_FAST,     // Clock out at 1 MHz
            .mode           = 0,                 // SPI mode 0
            .spics_io_num   = -1,                // CS pin
            .queue_size     = 8,                 // We want to be able to queue 7 transactions at a time
        };
        // Attach the LCD to the SPI bus
        ret = spi_bus_add_device(HSPI_HOST, &devcfg, spi);
        ESP_ERROR_CHECK(ret);
    }

    gpio_set_direction(IN_EXPANDER_CS, GPIO_MODE_OUTPUT);
    gpio_set_pull_mode(IN_EXPANDER_CS, GPIO_FLOATING);
    gpio_intr_disable(IN_EXPANDER_CS);

    spi_data(*spi, nullify, 64, IN_EXPANDER_CS);
    spi_data(*spi, inputdir, 3, IN_EXPANDER_CS);
    spi_data(*spi, invertpol, 3, IN_EXPANDER_CS);
}

uint8_t get_input_expander(spi_device_handle_t spi) {
    spi_transaction_t t;
    uint8_t           buffer[3] = {IN_EXPANDER_ADD | 1, IN_EXPANDER_GPIO_ADD, 0};
    esp_err_t         ret;

    memset(&t, 0, sizeof(t));
    t.length    = 8 * 3;
    t.tx_buffer = buffer;
    t.flags     = SPI_TRANS_USE_RXDATA;
    spi_device_acquire_bus(spi, portMAX_DELAY);
    gpio_set_level(IN_EXPANDER_CS, 0);
    ret = spi_device_polling_transmit(spi, &t);     // Transmit!
    gpio_set_level(IN_EXPANDER_CS, 1);
    spi_device_release_bus(spi);
    ESP_ERROR_CHECK(ret);

    return t.rx_data[2];
}
