#include <string.h>
#include "driver/spi_master.h"
#include "driver/gpio.h"
#include "system.h"
#include "output_expander.h"

void init_output_expander(spi_device_handle_t *spi, uint8_t add_spi_device) {
    esp_err_t ret;
    uint8_t   buffer[3] = {OUT_EXPANDER_ADD, OUT_EXPANDER_DIR_ADD, 0x00};
    uint8_t   nullify[64] = {0};

    if (add_spi_device) {
        spi_device_interface_config_t devcfg = {
            .clock_speed_hz = SPI_FREQ_FAST,
            .mode           = 0,                 // SPI mode 0
            .spics_io_num   = -1,                // OUT_EXPANDER_CS,     // CS pin
            .queue_size     = 8,                 // We want to be able to queue 7 transactions at a time
            //.pre_cb=lcd_spi_pre_transfer_callback,  //Specify pre-transfer callback to handle D/C line
        };
        // Attach the LCD to the SPI bus
        ret = spi_bus_add_device(HSPI_HOST, &devcfg, spi);
        ESP_ERROR_CHECK(ret);
    }

    gpio_set_direction(OUT_EXPANDER_CS, GPIO_MODE_OUTPUT);
    gpio_set_pull_mode(OUT_EXPANDER_CS, GPIO_FLOATING);
    gpio_intr_disable(OUT_EXPANDER_CS);

    spi_data(*spi, nullify, 64, OUT_EXPANDER_CS);
    spi_data(*spi, buffer, 3, OUT_EXPANDER_CS);
}

void set_output_expander(spi_device_handle_t spi, uint8_t map) {
    esp_err_t         ret;
    spi_transaction_t t;
    uint8_t           buffer[3] = {0};
    int counter = 0;

    memset(&t, 0, sizeof(t));     // Zero out the transaction
    t.length    = 8 * 3;          // Command is 8 bits
    buffer[0]   = OUT_EXPANDER_ADD;
    buffer[1]   = OUT_EXPANDER_GPIO_ADD;
    buffer[2]   = map;
    t.tx_buffer = buffer;     // The data is the cmd itself
    do {
        spi_device_acquire_bus(spi, portMAX_DELAY);
        gpio_set_level(OUT_EXPANDER_CS, 0);
        ets_delay_us(1);
        ret = spi_device_polling_transmit(spi, &t);     // Transmit!
        gpio_set_level(OUT_EXPANDER_CS, 1);
        spi_device_release_bus(spi);
    } while (ret != ESP_OK && counter++ < 10);
    //ESP_ERROR_CHECK(ret);
    //FIXME: Cannot send polling transaction while the previous polling transaction is not terminated!
}
