#include <string.h>
#include "state.h"
#include "adc_spi.h"
#include "dac_spi.h"
#include "input_expander.h"
#include "output_expander.h"
#include "debounce.h"
#include "output_expander.h"
#include "input_expander.h"
#include "adc_spi.h"
#include "dac_spi.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "freertos/timers.h"
#include "system.h"
#include "driver/uart.h"
#include "uart_primary.h"
#include "utility.h"
#include "serial485.h"
#include "debounce.h"
#include "asynctasks.h"
#include "esp_task_wdt.h"
#include "packet_manager.h"

#define SAMPLES_NUM 10
#define AVERAGE_NUM 1

#define TAG     "state"

static digin_filter_t filter;
static uint16_t adc1_samples[AVERAGE_NUM];
static uint16_t adc2_samples[AVERAGE_NUM];
static uint16_t press1_samples[AVERAGE_NUM];
static uint16_t press2_samples[AVERAGE_NUM];
static uint16_t temp1_samples[AVERAGE_NUM];
static uint16_t temp2_samples[AVERAGE_NUM];
static uint16_t ma1_samples[AVERAGE_NUM];
static uint16_t ma2_samples[AVERAGE_NUM];
static int samples_index = 0;
static QueueHandle_t  update_queue;

//static uint16_t *samples_pool[ADC_CH_NUM] = {adc1_samples, adc2_samples, press1_samples, press2_samples, ma1_samples, ma2_samples, temp1_samples, temp2_samples};
static uint16_t *samples_pool[ADC_CH_NUM] = {temp1_samples, temp2_samples, press1_samples, press2_samples, ma1_samples, ma2_samples, adc1_samples, adc2_samples};

static SemaphoreHandle_t access_state_sem;
static machine_state_t internal_state;

void update_state(uint8_t outputs, uint8_t dac1, uint8_t dac2, spi_device_handle_t out, spi_device_handle_t dac) {
    set_output_expander(out, outputs); 
    set_dac_channel(dac, DAC_CH_0,  volts_to_dac(dac1));
    set_dac_channel(dac, DAC_CH_1,  volts_to_dac(dac2));
}

//TODO: gestire gli input resilienti anche per gli slave
uint8_t get_local_inputs(digin_filter_t *filt) {
    uint8_t res = 0x00;
    uint8_t value;
    int i;

    for (i = 0; i < 8; i++) {
        value = digital_read(i, filt);
        res |= value << i;
    }

    return res;
}

int check_alarm(int alarm) {
    uint8_t bitmap[TOT_INPUTS];
    int index, shift;

    if (alarm < 0 || alarm >= TOT_INPUTS*8)
        return 0;
        
    get_input_state(bitmap);
    index = alarm/8;
    shift = alarm % 8;
    return !((bitmap[index] >> shift) & 0x01);
}

void reset_resilient_inputs() {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    internal_state.localinputs = internal_state.rawlocalinputs;

    for (i = 0; i < TOT_EXT_INPUTS; i++)
        internal_state.externinputs[i] = internal_state.rawexterninputs[i];

    xSemaphoreGive(access_state_sem);
}

int queue_adc_value(uint16_t *buffer, uint16_t value, int max, int index) {
    buffer[index] = value;
    return (index+1) % max;
}

unsigned int average_adc_value(uint16_t *buffer, int max) {
    unsigned int res = 0, i;
    for (i = 0; i < max; i++) {
        res += buffer[i];
    }

    return res/max;
}

void init_state_manager(machine_configuration_t *config) {
    int i;
    memset(&internal_state, 0, sizeof(machine_state_t));
    for (i = 0; i < NUM_DAC; i++)
        internal_state.localdac[i] = 50;
    for (i = 0; i < TOT_EXT_DAC; i++)
        internal_state.externdac[i] = 50;
    access_state_sem = xSemaphoreCreateMutex();
    internal_state.colore[0] = config->colore[0];
    internal_state.colore[1] = config->colore[1];
    init_digin_filter(&filter, (uint16_t)config->inputs[0], 0x0000);
}

void init_digin(machine_configuration_t *config) {
    init_digin_filter(&filter, (uint16_t)config->inputs[0], 0x0000);
}
    spi_device_handle_t io_expander, spi_adc, spi_dac;


void task_main_cycle(void *args) {
    void **params = (void**)args;
    QueueHandle_t       *queue = (QueueHandle_t*)params[0];
    machine_configuration_t *config = (machine_configuration_t*) params[1];
    EventGroupHandle_t *eventgroup = (EventGroupHandle_t*) params[2];
    internal_state_message_t message;
    int i, j, newindex, result;
    //int received, slave = 0, oldslave, len;
    //uint8_t buffer485[256];
    uint8_t value = 0x00, oldreset;
    uint8_t out, dac1, dac2;
    uint8_t bitmap[TOT_OUTPUTS+TOT_INPUTS];
    uint16_t buffer420ma[TOT_420], ma420tmp1, ma420tmp2;
    uint32_t averageadc = 0;
    char flag;
    unsigned long timestamp, timestamp485, inputtime, timestampreset, timestamp420;
    //TickType_t lastwaketime;
    colori_led_t oldcolors[NUM_SLAVES];

    update_queue = xQueueCreate(256, sizeof(internal_state_message_t));

    //init_485_generic();
    init_spi();
    init_output_expander(&io_expander, 1);
    init_input_expander(&io_expander, 0);
    init_adc(&spi_adc);
    init_dac(&spi_dac);

    set_output_expander(io_expander, 0x0);
    set_dac_channel(spi_dac, DAC_CH_0, 0);
    set_dac_channel(spi_dac, DAC_CH_1, 127);

    esp_task_wdt_add(NULL);
    inputtime = timestamp = timestamp485 = timestampreset = timestamp420 = get_millis();

    for(;;) {
        esp_task_wdt_reset();
        //lastwaketime = xTaskGetTickCount();

        while (xQueueReceive(update_queue, &message, 5/portTICK_PERIOD_MS)) {
            if (!internal_state.reset) {
                manage_message(message);
                // Update the local state
                xSemaphoreTake(access_state_sem, WAIT4SEM);
                out = internal_state.localoutputs;
                dac1 = internal_state.localdac[0];
                dac2 = internal_state.localdac[1];
                xSemaphoreGive(access_state_sem);
                update_state(out, dac1, dac2, io_expander, spi_dac);

                xQueueSend(*queue, NULL, 0);
            }
        }

        if (get_millis() - timestamp420 >= 15) {
            ma420tmp1 = scale_to(get_adc_channel(spi_adc, ADC_CH_420MA1), 4095, 1023);
            ma420tmp2 = scale_to(get_adc_channel(spi_adc, ADC_CH_420MA2), 4095, 1023);
            xSemaphoreTake(access_state_sem, WAIT4SEM);
            if (ma420tmp1 != internal_state.local420[0] || ma420tmp2 != internal_state.local420[1]) {
                internal_state.local420[0] = ma420tmp1;
                internal_state.local420[1] = ma420tmp2;
                flag = 1;
            }
            else {
                flag = 0;
            }
            xSemaphoreGive(access_state_sem);
            if (flag)
                xEventGroupSetBits(*eventgroup, ADC_EVENT_BITS);
            
            timestamp420 = get_millis();
        }


        if (get_millis() > 100 && get_millis() - timestampreset >= 20) {
        //if (get_millis() > 100 && xQueueReceive(*queue, &message, 0)) { //TODO: rimuovi la coda, non serve a un granche'
            oldreset = internal_state.reset;
            internal_state.reset = 0;//check_alarm(config->alarm);
            if (internal_state.reset) {
                // L'emergenza non deve resettare le uscite del primo blocco
                bitmap[0] = 0x00;
                memset(&bitmap[1], 0xFF, TOT_OUTPUTS-1);
                // Cancella i task che hanno uscite su degli slave
                delete_tasks_by_output(bitmap);

                get_output_state(bitmap);
                if (oldreset != internal_state.reset) {
                    get_color_state((uint8_t*)oldcolors);
                    memcpy(internal_state.outputs_before_reset, bitmap, TOT_OUTPUTS);
                }

                set_bitmap(bitmap, &config->reset_outputs, 1);
                memset(&bitmap[1], 0, TOT_OUTPUTS-1);
                // Metti a zero tutte le uscite che non sono sul master e attiva le uscite di emergenza
                update_output_state(bitmap);
                for (i = 0; i < NUM_SLAVES; i++)
                    bitmap[i] = ROSSO;
                set_color_state(bitmap);
            }
            else {
                // Ripristina le uscite dopo il reset
                if (oldreset != internal_state.reset) {
                    update_output_state(internal_state.outputs_before_reset);
                    set_color_state((uint8_t*)oldcolors);
                }

                // Controlla gli allarmi
                get_input_state(bitmap);
                xSemaphoreTake(access_state_sem, WAIT4SEM);
                memcpy(buffer420ma, internal_state.local420, sizeof(uint16_t)*NUM_420);
                memcpy(&buffer420ma[NUM_420], internal_state.extern420, sizeof(uint16_t)*NUM_420);
                xSemaphoreGive(access_state_sem);
                delete_tasks_by_alarm(bitmap, buffer420ma);
            }

            // Update the local state
            xSemaphoreTake(access_state_sem, WAIT4SEM);
            out = internal_state.localoutputs;
            dac1 = internal_state.localdac[0];
            dac2 = internal_state.localdac[1];
            xSemaphoreGive(access_state_sem);
            update_state(out, dac1, dac2, io_expander, spi_dac);

            timestampreset = get_millis();
            xQueueReset(*queue);
        }

        // Leggi gli ingressi
        if (get_millis() - inputtime >= 5) {
            value = get_input_expander(io_expander);
            result = digin_filter(&filter, value, 4);
            // Se c'e' un aggiornamento procedere di conseguenza
            //if (digin_filter(&filter, value, 4) || digital_read(config->alarm, &filter)) {
            xSemaphoreTake(access_state_sem, WAIT4SEM);
            internal_state.rawlocalinputs = get_local_inputs(&filter);
            bitmap[0] = internal_state.rawlocalinputs;
            for (i = 0; i < TOT_EXT_INPUTS; i++)
                bitmap[1+i] = internal_state.rawexterninputs[i];
            xSemaphoreGive(access_state_sem);
            update_input_state(bitmap, config->specialinputs, config->invertedinputs);
            xQueueSend(*queue, NULL, 0);
            if (result)
                xEventGroupSetBits(*eventgroup, INPUT_EVENT_BITS);
            inputtime = get_millis();
        }

        // Read and elaborate average for analog values
        if (get_millis() - timestamp >= 100) {
            for (i = 0; i < ADC_CH_NUM; i++) {
                // I 420ma vengono letti piu' velocemente
                if (i == ADC_CH_420MA1 || i == ADC_CH_420MA2)
                    continue;

                averageadc = 0;
                for (j = 0; j < SAMPLES_NUM; j++)
                    averageadc += get_adc_channel(spi_adc, i);

                averageadc /= SAMPLES_NUM;
                newindex = queue_adc_value(samples_pool[i], averageadc, AVERAGE_NUM, samples_index);
            }

            xSemaphoreTake(access_state_sem, WAIT4SEM);
            internal_state.localadc[0] = average_adc_value(adc1_samples, AVERAGE_NUM);
            internal_state.localadc[1] = average_adc_value(adc2_samples, AVERAGE_NUM);
            internal_state.localtemp[0] = average_adc_value(temp1_samples, AVERAGE_NUM);
            internal_state.localtemp[1] = average_adc_value(temp2_samples, AVERAGE_NUM);
            internal_state.localpress[0] = average_adc_value(press1_samples, AVERAGE_NUM);
            internal_state.localpress[1] = average_adc_value(press2_samples, AVERAGE_NUM);
            //internal_state.local420[0] = scale_to(average_adc_value(ma1_samples, AVERAGE_NUM), 4095, 1023);
            //internal_state.local420[1] = scale_to(average_adc_value(ma2_samples, AVERAGE_NUM), 4095, 1023);
            samples_index = newindex;
            xSemaphoreGive(access_state_sem);
            timestamp = get_millis();
            xEventGroupSetBits(*eventgroup, ADC_EVENT_BITS);
        }

        // Aggiorna gli slave
        /*if (get_millis() - timestamp485 >= 30) {
            int tmp;
            
            //FIXME: mettiti d'accordo con michele su come gestire la configurazione, per ora vuole che lo slave 1 sia sempre attivo
            if (slave == 0 || is_slave_configured(config, slave)) {
                get_output_state(bitmap);
                turn_on_led(*config, bitmap);
                set_output_state(bitmap);
                len = pack_slave_state(buffer485, config->f_pulsing_led ? config->pulsing_led : 0, slave);
                send485_state(slave, buffer485, len);
                memset(buffer485, 0, SLAVE_STATE_LEN);
                received = receive485(buffer485, SLAVE_STATE_LEN, 2);
                //hexprint(buffer485, received, TAG);
                if (received == 0) {
                    //TODO: verifica che la configurazione escluda correttamente lo slave
                    ESP_LOGI(TAG, "no answer to %i bytes from %i", len, slave);
                } else {
                    if ((tmp = validate_slave_state(buffer485, received, slave)) == 0) {
                        // se lo stato e' cambiato notifica i processi
                        if (unpack_slave_state(&buffer485[2], slave) || 1)
                            xEventGroupSetBits(*eventgroup, ADC_EVENT_BITS | INPUT_EVENT_BITS);
                    }
                    else
                        ESP_LOGI(TAG, "Invalid response of %i bytes: %i", received, tmp);
                    xQueueSend(*queue, NULL, 0);
                }
            }

            oldslave = slave;
            do {
                slave = (slave+1) % NUM_SLAVES;
            } while (slave != oldslave && !is_slave_configured(config, slave));
            timestamp485 = get_millis();
        }*/

        //vTaskDelay(5/portTICK_PERIOD_MS);
    }
}

void take_offset_press(machine_configuration_t *config) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    config->offsetpress[0] = internal_state.localpress[0];
    config->offsetpress[1] = internal_state.localpress[1];
    for (i = 0; i < TOT_EXT_PRESS; i++)
        config->offsetpress[2+i] = internal_state.externpress[i];
    xSemaphoreGive(access_state_sem);
}

//WRITE
void update_output_state(uint8_t *bitmap) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    internal_state.localoutputs = bitmap[0];
    for(i = 0; i < TOT_EXT_OUTPUTS; i++) {
        internal_state.externoutputs[i] = bitmap[1+i];
    }
    xSemaphoreGive(access_state_sem);
}

void set_output_state(uint8_t *bitmap) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    internal_state.localoutputs |= bitmap[0];
    for(i = 0; i < TOT_EXT_OUTPUTS; i++) {
        internal_state.externoutputs[i] |= bitmap[1+i];
    }
    xSemaphoreGive(access_state_sem);
}


void clear_output_state(uint8_t* bitmap) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    internal_state.localoutputs &= ~bitmap[0];
    for(i = 0; i < TOT_EXT_OUTPUTS; i++) {
        internal_state.externoutputs[i] &= ~bitmap[1+i];
    }
    xSemaphoreGive(access_state_sem);
}

void update_input_state(uint8_t *bitmap, uint8_t *resilient, uint8_t *inverted) {
    int i, j;
    xSemaphoreTake(access_state_sem, WAIT4SEM);

    for (j = 0; j < 8; j++) {
        if ((resilient[0] & (1 << j)) && ((bitmap[0] & (1 << j)) == 0))
            continue;
        else if (bitmap[0] & (1 << j))
            internal_state.localinputs |= (1 << j);
        else 
            internal_state.localinputs &= ~(1 << j);
    }
    internal_state.localinputs ^= inverted[0];

    for(i = 0; i < TOT_EXT_INPUTS; i++) {
        for (j = 0; j < 8; j++) {
            if ((resilient[i+1] & (1 << j)) && ((bitmap[i+1] & (1 << j)) == 0))
                continue;
            else if (bitmap[i+1] & (1 << j))
                internal_state.externinputs[i] |= (1 << j);
            else 
                internal_state.externinputs[i] &= ~(1 << j);
        }
        internal_state.externinputs[i] ^= inverted[i+1];
        //ESP_LOGI(TAG, "raw vs actual: %02X - %02X", internal_state.rawexterninputs[0], internal_state.externinputs[0]);
    }
    xSemaphoreGive(access_state_sem);
}


int update_dac_state(uint8_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for(i = 0; i < NUM_DAC; i++)
        internal_state.localdac[i] = buffer[i];
    for(i = 0; i < TOT_EXT_DAC; i++)
        internal_state.externdac[i] = buffer[NUM_DAC+i];
    xSemaphoreGive(access_state_sem);

    return TOT_DAC;
}

void update_single_dac_state(int num, uint8_t value) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    if (num < NUM_DAC)
        internal_state.localdac[num] = value;
    else 
        internal_state.externdac[num-NUM_DAC] = value;
    xSemaphoreGive(access_state_sem);
}

void set_activity_bitmap_output(uint8_t *bitmap) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    set_bitmap(internal_state.activitybitmap, bitmap, TOT_OUTPUTS);
    xSemaphoreGive(access_state_sem);
}

void clear_activity_bitmap_output(uint8_t *bitmap) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    clear_bitmap(internal_state.activitybitmap, bitmap, TOT_OUTPUTS);
    xSemaphoreGive(access_state_sem);
}

void clear_activity_bitmap_dac(uint8_t bitmap) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    clear_bitmap(&internal_state.activitydac, &bitmap, 1);
    xSemaphoreGive(access_state_sem);
}

void set_activity_bitmap_dac(uint8_t bitmap) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    set_bitmap(&internal_state.activitydac, &bitmap, 1);
    xSemaphoreGive(access_state_sem);
}

void set_color_state(uint8_t *buffer) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    internal_state.colore[0] = buffer[0];
    internal_state.colore[1] = buffer[1];
    xSemaphoreGive(access_state_sem);
}

int pack_slave_state(uint8_t *buffer, uint8_t pulsingled, int slave) {
    int i = 0, j = 0, sub;
    sub = NUM_OUTPUTS*8 + slave*NUM_EXT_OUTPUTS*8;
    if (sub <= pulsingled)
        pulsingled = pulsingled + 1 - sub;
    else 
        pulsingled = 0;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (j = slave*NUM_EXT_OUTPUTS; j < (slave+1)*NUM_EXT_OUTPUTS; j++)
        buffer[i++] = internal_state.externoutputs[j];
    for (j = slave*NUM_EXT_DAC; j < (slave+1)*NUM_EXT_DAC; j++)
        buffer[i++] = internal_state.externdac[j];
    buffer[i++] = internal_state.colore[slave];
    buffer[i++] = pulsingled;
    xSemaphoreGive(access_state_sem);
    return i;
}

int unpack_slave_state(uint8_t *buffer, int slave) {
    int i = 0, j = 0, result;
    machine_state_t tmpstate;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    memcpy((uint8_t*)&tmpstate, (uint8_t*)&internal_state, sizeof(machine_state_t));
    internal_state.rawexterninputs[slave] = buffer[i++];
    for (j = 0; j < NUM_EXT_TEMP; j++) {
        internal_state.externtemp[NUM_EXT_TEMP*slave+j] = buffer[i] << 8 | buffer[i+1];
        i += 2;
    }
    for (j = 0; j < NUM_EXT_PRESS; j++) {
        internal_state.externpress[NUM_EXT_PRESS*slave+j] = buffer[i] << 8 | buffer[i+1];
        i += 2;
    }
    for (j = 0; j < NUM_EXT_ADC; j++) {
        internal_state.externadc[NUM_EXT_ADC*slave+j] = buffer[i] << 8 | buffer[i+1];
        i += 2;
    }
    for (j = 0; j < NUM_EXT_420; j++) {
        uint16_t intermezzo;
        intermezzo = buffer[i] << 8 | buffer[i+1];
        internal_state.extern420[NUM_EXT_420*slave+j] = intermezzo;
        i += 2;
    }
    result = memcmp((uint8_t*)&tmpstate, (uint8_t*)&internal_state, sizeof(machine_state_t));
    xSemaphoreGive(access_state_sem);
    return result;
}

// READ
uint8_t get_single_output_state(int num) {
    uint8_t res;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    res = num == 0 ? internal_state.localoutputs : internal_state.externoutputs[num-1];
    xSemaphoreGive(access_state_sem);
    return res;
}

void get_output_state(uint8_t *bitmap) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    bitmap[0] = internal_state.localoutputs;
    for(i = 0; i < TOT_EXT_OUTPUTS; i++)
        bitmap[1+i] = internal_state.externoutputs[i];
    xSemaphoreGive(access_state_sem);
}

int get_input_state(uint8_t *bitmap) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    bitmap[0] = internal_state.localinputs;
    for(i = 0; i < TOT_EXT_INPUTS; i++)
        bitmap[1+i] = internal_state.externinputs[i];
    xSemaphoreGive(access_state_sem);
    return TOT_INPUTS;
}

int get_temps_state(uint8_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_TEMP; i++)
        buffer[i] = adc_to_temp(internal_state.localtemp[i]);
    for (i = 0; i < TOT_EXT_TEMP; i++)
        buffer[NUM_TEMP+i] = adc_to_temp(internal_state.externtemp[i]);
    xSemaphoreGive(access_state_sem);
    return TOT_TEMP;
}

int get_press_state(uint8_t *buffer, uint16_t *offsets) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_PRESS; i++)
        buffer[i] = adc_to_bar(internal_state.localpress[i], offsets[i]);
    for (i = 0; i < TOT_EXT_PRESS; i++)
        buffer[NUM_PRESS+i] = adc_to_bar(internal_state.externpress[i], offsets[NUM_PRESS+i]);
    xSemaphoreGive(access_state_sem);
    return TOT_PRESS;
}

int get_adc_state(uint8_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_ADC; i++)
        buffer[i] = adc_to_10v(internal_state.localadc[i]);
    for (i = 0; i < TOT_EXT_ADC; i++)
        buffer[NUM_ADC+i] = adc_to_10v(internal_state.externadc[i]);
    xSemaphoreGive(access_state_sem);
    return TOT_ADC;
}

int get_420ma_state(uint8_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_420; i++)
        buffer[i] = adc_to_420ma(internal_state.local420[i]);
    for (i = 0; i < TOT_EXT_420; i++)
        buffer[NUM_420+i] = adc_to_420ma(internal_state.extern420[i]);
    xSemaphoreGive(access_state_sem);
    return TOT_420;
}

int get_pure420ma_state(uint16_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_420; i++)
        buffer[i] = internal_state.local420[i];
    for (i = 0; i < TOT_EXT_420; i++)
        buffer[NUM_420+i] = internal_state.extern420[i];
    xSemaphoreGive(access_state_sem);
    return TOT_420;
}

int get_dac_state(uint8_t *buffer) {
    int i;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    for (i = 0; i < NUM_DAC; i++)
        buffer[i] = internal_state.localdac[i];
    for (i = 0; i < TOT_EXT_DAC; i++)
        buffer[NUM_DAC+i] = internal_state.externdac[i];
    xSemaphoreGive(access_state_sem);
    return TOT_DAC;
}

void get_color_state(uint8_t *buffer) {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    buffer[0] = internal_state.colore[0];
    buffer[1] = internal_state.colore[1];
    xSemaphoreGive(access_state_sem);
}

int check_activity_bitmap_output(uint8_t *buffer) {
    int res;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    res = check_bitmap_intersection(internal_state.activitybitmap, buffer, TOT_OUTPUTS);
    xSemaphoreGive(access_state_sem);
    return res;
}

int check_activity_bitmap_dac(uint8_t buffer) {
    int res;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    res = check_bitmap_intersection(&internal_state.activitydac, &buffer, 1);
    xSemaphoreGive(access_state_sem);
    return res;
}

int pack_state(uint8_t *buffer, uint16_t *offsets) {
    int i, j;
    j = 0;
    xSemaphoreTake(access_state_sem, WAIT4SEM);
    buffer[j++] = internal_state.localinputs;
    for (i = 0; i < NUM_EXT_INPUTS*2; i++)
        buffer[j++] = internal_state.externinputs[i];
    buffer[j++] = internal_state.localoutputs;
    for (i = 0; i < NUM_EXT_OUTPUTS*2; i++)
        buffer[j++] = internal_state.externoutputs[i];
    for (i = 0; i < NUM_TEMP; i++)
        buffer[j++] = adc_to_temp(internal_state.localtemp[i]);
    for (i = 0; i < NUM_EXT_TEMP*2; i++)
        buffer[j++] = adc_to_temp(internal_state.externtemp[i]);
    for (i = 0; i < NUM_PRESS; i++)
        buffer[j++] = adc_to_bar(internal_state.localpress[i], offsets[i]);
    for (i = 0; i < NUM_EXT_PRESS*2; i++)
        buffer[j++] = adc_to_bar(internal_state.externpress[i], offsets[NUM_PRESS+i]);
    for (i = 0; i < NUM_DAC; i++)
        buffer[j++] = internal_state.localdac[i];
    for (i = 0; i < NUM_EXT_DAC*2; i++)
        buffer[j++] = internal_state.externdac[i];
    for (i = 0; i < NUM_ADC; i++)
        buffer[j++] = adc_to_10v(internal_state.localadc[i]);
    for (i = 0; i < TOT_EXT_ADC; i++)
        buffer[j++] = adc_to_10v(internal_state.externadc[i]);
    for (i = 0; i < NUM_420; i++)
        buffer[j++] = adc_to_420ma(internal_state.local420[i]);
    for (i = 0; i < NUM_EXT_420*2; i++)
        buffer[j++] = adc_to_420ma(internal_state.extern420[i]);

    xSemaphoreGive(access_state_sem);
    return j;
}

//TODO: risolvi per bene questo problema (se e' un problema)
void takeStateSemaphore() {
    xSemaphoreTake(access_state_sem, WAIT4SEM);
}

void giveStateSemaphore() {
    xSemaphoreGive(access_state_sem);
}

void manage_message(internal_state_message_t message) {
    if (message.notvalid)
        return;

    switch(message.type) {
        case STATE_SET_OUTPUTS:
            set_output_state(message.bitmap);
            break;
        case STATE_CLEAR_OUTPUTS:
            clear_output_state(message.bitmap);
            break;
        case STATE_UPDATE_OUTPUTS:
            update_output_state(message.bitmap);
            break;
        case STATE_CLEAR_ACTIVITY_OUTPUTS:
            clear_activity_bitmap_output(message.bitmap);
            break; 
        case STATE_CLEAR_ACTIVITY_DAC:
            clear_activity_bitmap_dac(message.bitmap[0]);
            break; 
        case STATE_SET_ACTIVITY_OUTPUTS:
            set_activity_bitmap_output(message.bitmap);
            break; 
        case STATE_SET_ACTIVITY_DAC:
            set_activity_bitmap_dac(message.bitmap[0]);
            break; 
        case STATE_UPDATE_DAC:
            update_single_dac_state(message.num, message.value);
            break;
    }
}

void send_output_state(internal_state_message_type_t type, uint8_t *bitmap, int notvalid) {
    internal_state_message_t message;
    memcpy(message.bitmap, bitmap, TOT_OUTPUTS);
    message.notvalid = notvalid;
    message.type = type;
    xQueueSend(update_queue, &message, 0);
}

void send_dac_state(int value, int num, int notvalid) {
    internal_state_message_t message;
    message.notvalid = notvalid;
    message.num = num;
    message.value = value;
    message.type = STATE_UPDATE_DAC;
    xQueueSend(update_queue, &message, 0);
}

void send_clear_activity_dac(uint8_t dac, int notvalid) {
    internal_state_message_t message;
    message.notvalid = notvalid;
    message.bitmap[0] = dac;
    message.type = STATE_CLEAR_ACTIVITY_DAC;
    xQueueSend(update_queue, &message, 0);
}

int is_in_reset() {
    return internal_state.reset;
}