#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/timers.h"
#include "esp_log.h"

unsigned long get_millis() {
    return (unsigned long) xTaskGetTickCount()*portTICK_PERIOD_MS;
}

uint16_t scale_to(uint16_t value, uint16_t max, uint16_t scale) {
    uint32_t tmp = value * scale;
    tmp = tmp/max;
    return (uint16_t)tmp;
}

uint8_t custom_crc(uint8_t *data, int len, uint8_t start) {
    uint16_t buffer = start;
    int     i;

    for (i = 0; i < len; i++)
        buffer = (buffer + data[i]) % 256;

    return (uint8_t) buffer;
}

uint8_t simple_crc(uint8_t *data, int len) {
    uint16_t buffer = 0;
    //uint8_t res = 0xFF;
    int     i;

    for (i = 0; i < len; i++) {
        buffer = (buffer + data[i]) % 256;
        //res ^= data[i];
    }

    //return res;
    return (uint8_t) buffer;
}

void hexprint(uint8_t *data, int len, char* tag) {
    int i = 0;
    //char *string = (char*) malloc(sizeof(char)*3*len+1);
    char string[256];
    memset(string, 0, 3*len+1);

    for (i = 0; i < len; i++) {
        sprintf(&string[i*3], "%02X-", data[i]);
    }

    ESP_LOGI(tag, "%s", string);
    //free(string);
}


void set_single_bitmap(int num, uint8_t *buffer) {
    int i = num/8;
    int shift = num%8;
    buffer[i] |= 1 << shift;
}

int check_bitmap_intersection(uint8_t *bitmap1, uint8_t *bitmap2, int len) {
    int i;
    for (i = 0; i < len; i++)
        if (bitmap1[i] & bitmap2[i])
            return 1;
    return 0;
}

int check_bitmap_inclusion(uint8_t *bitmap1, uint8_t *bitmap2, int len) {
    int i;
    for (i = 0; i < len; i++)
        if (bitmap1[i] & ~bitmap2[i])
            return 0;
    return 1;
}

void set_bitmap(uint8_t *dest, uint8_t *setunion, int len) {
    int i;
    for (i = 0; i < len; i++)
        dest[i] |= setunion[i];
}

void clear_bitmap(uint8_t *dest, uint8_t *intersection, int len) {
    int i;
    for (i = 0; i < len; i++)
        dest[i] &= ~intersection[i];
}

void shift_array_left(uint8_t *array, int len, int shift) {
    int i;
    for (i = 0; i < len-shift; i++)
        array[i] = array[i+shift];
}