#!/usr/bin/env python3.5
import sys
import glob
import argparse
import serial

# UTILITY

def simpleCrc(data):
    crc = 0xFF
    buf = 0
    for el in data:
#crc = crc ^ el
        buf = (buf + el) % 256
    return buf

# SERIAL

def serial_ports():
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result

def sendPacket(command, data, ser):
    packet = [2, 1, len(data)+8, 0, 1, 1, 0, 2, 2,
              command >> 8, command & 0xFF] + data
    packet.append(simpleCrc(packet[3:]))
    ser.write(bytearray(packet))
    return packet

def connect(port):
    serialPort = serial.Serial(port=port, baudrate=115200, timeout=0.020)
    serialPort.flushInput()
    serialPort.flushOutput()
    return serialPort

def hexlist(bytelist):
    return [hex(int.from_bytes([x], "big")) for x in bytelist]

# MAIN

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--command", type=str, required=True, help="Codice del comando da inviare (2 byte, in esadecimale)")
    parser.add_argument("-p", "--port", type=str, required=False, default="/dev/ttyUSB0", help="Porta seriale da usare (default /dev/ttyUSB0)")
    parser.add_argument("-d", "--deaf", action='store_true', required=False, help="Flag che indica di non ascoltare la risposta")
    parser.add_argument('data', nargs='*')
    args = parser.parse_args()

    data = []
    for el in args.data:
        if el.startswith("0d"):
            data.append(int(el.replace("0d", ""),10))
        elif el.startswith("0b"):
            data.append(int(el.replace("0b", ""),2))
        elif el.startswith("0x"):
            data.append(int(el.replace("0x", ""),16))
        else:
            data.append(int(el,16))

    port = connect(args.port)
    print("pacchetto inviato:")
    print(sendPacket(int(args.command, 16), data, port))

    if not args.deaf:
        ack = port.read(255)

        if len(ack) > 0:
            print("Contenuto ricevuto: {}".format(len(ack)))
            print("Lunghezza = {}\nComando = {}\nData = {}".format(ack[2], hex(int.from_bytes(ack[9:11], "big")), hexlist(ack[11:len(ack)-1])))
            print(["{} ({})".format(int.from_bytes([x], "big"), chr(x)) for x in list(ack)])
        else:
            print("Nessuna risposta")
