#!/usr/bin/env python3.5
import sys
import glob
import argparse
import serial
import random
from time import sleep


# UTILITY

def simpleCrc(data):
    crc = 0xFF
    buf = 0
    for el in data:
        #crc = crc ^ el
        buf = (buf + el) % 256
    return buf

# SERIAL


def serial_ports():
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result


def sendPacket(command, data, ser):
    packet = [2, 1, len(data)+8, 0, 1, 1, 0, 2, 2,
              command >> 8, command & 0xFF] + data
    packet.append(simpleCrc(packet[3:]))
    ser.write(bytearray(packet))
    return packet


def connect(port):
    serialPort = serial.Serial(port=port, baudrate=115200, timeout=0.020)
    serialPort.flushInput()
    serialPort.flushOutput()
    return serialPort


def hexlist(bytelist):
    return [hex(int.from_bytes([x], "big")) for x in bytelist]


def randombyte():
    powers = [0] + [2**x for x in range(7)]
    x = random.choice(powers)
    powers.remove(x)
    return x + random.choice(powers)


def randombytes(len):
    return [randombyte() for x in range(len)]


def randomoutputs():
    powers = [2**x for x in range(23)]
    x = random.choice(powers)
    return [(x >> 16) & 0xFF, (x >> 8) & 0xFF, x & 0xFF]


def nodata():
    return []


def outputstime():
    return [0, 0] + randomoutputs() + [0, 50]


def outputstimeafterinput():
    return [random.randint(0, 7), 0, 50, 0, random.randint(41, 200), random.randint(0, 7), random.randint(0, 7)]


def outputstimedacreact():
    return [0, 0] + randomoutputs() + [0, random.randint(10, 100), 2, random.randint(41, 200), 0, 20, 60, random.randint(41, 200)]


def outputstime420ma():
    return [0, 0] + randomoutputs() + [0, random.randint(10, 100), 1, 80]

# MAIN


if __name__ == '__main__':
    COMMANDS = [
        #{"code": 0x1000, "name": "GET_STATE", "data": nodata},
        {"code": 0x2003, "name": "SET_OUTPUTS_TIME", "data": outputstime},
        {"code": 0x201E, "name": "SET_OUTPUTn_TIME_AFTER_INPUT",
            "data": outputstimeafterinput},
        {"code": 0x2018, "name": "SET_OUTPUTS_TIME_420MA_DAC_REACT",
            "data": outputstimedacreact},
        {"code": 0x2011, "name": "SET_OUTPUTS_TIME_420MA", "data": outputstime420ma},
    ]

    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--port", type=str, required=False,
                        default="/dev/ttyUSB0", help="Porta seriale da usare (default /dev/ttyUSB0)")
    parser.add_argument("-i", "--interval", type=int, required=False, default="50",
                        help="millisecondi (indicativi) tra un comando e il seguente")
    parser.add_argument("-d", "--deaf", action='store_true', required=False,
                        help="Flag che indica di non ascoltare la risposta")
    args = parser.parse_args()

    port = connect(args.port)
    interval = args.interval/1000

    while True:
        command = random.choice(COMMANDS)
        print("invio comando {}".format(command["name"]))
        sendPacket(command["code"], command["data"](), port)

        if not args.deaf:
            ack = port.read(255)
            if len(ack) == 0:
                print("Nessuna risposta!")
                exit(1)

        sleep(interval)
